

package com.lutao.entity;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.lutao.annotate.Validation;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.time.LocalDateTime;

/**
 * 气象监测信息
 *
 * @author guoyc
 * @date 2023-07-20 10:56:48
 */
@Data
@TableName("atmosphere_monitor_info")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "气象监测信息")
public class AtmosphereMonitorInfo extends Model<AtmosphereMonitorInfo> {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    @TableId
    @ApiModelProperty(value="主键")
    private Integer id;

    /**
     * 海域名称
     */
    @ApiModelProperty(value="海域名称")
    private String nameWaters;

    /**
     * 预报时效
     */
    @ApiModelProperty(value="预报时效")
    private String forecastEfficiency;

    /**
     * 系统平台id
     */
    @ApiModelProperty(value="系统平台id")
    private String pfId;

    /**
     * 应用id
     */
    @ApiModelProperty(value="应用id")
    private String appId;

    /**
     * 平台编码
     */
    @ApiModelProperty(value="平台编码")
    @Validation(message = "atmosphere_monitor_info表{platform_code}不能为空")
    private String platformCode;

    /**
     * 平台名称
     */
    @ApiModelProperty(value="平台名称")
    private String platformName;

    /**
     * 平台id
     */
    @ApiModelProperty(value="平台id")
    private String platformId;

    /**
     * 风速度
     */
    @ApiModelProperty(value="风速度")
    //@Validation(message = "atmosphere_monitor_info表{wind_speed}不能为空")
    private String windSpeed;

    /**
     * 风向
     */
    @ApiModelProperty(value="风向")
    //@Validation(message = "atmosphere_monitor_info表{wind_direction}不能为空")
    private String windDirection;

    /**
     * 阵风
     */
    @ApiModelProperty(value="阵风")
    private String gustiness;

    /**
     * 气压
     */
    @ApiModelProperty(value="气压")
    //@Validation(message = "atmosphere_monitor_info表{pressure}不能为空")
    private Float pressure;

    /**
     * 浪高
     */
    @ApiModelProperty(value="浪高")
    private Float waveHeight;

    /**
     * 最大波高
     */
    @ApiModelProperty(value="最大波高")
    private String maxWaveHeight;

    /**
     * 海浪方向
     */
    @ApiModelProperty(value="海浪方向")
    private String waveDirection;

    /**
     * 周期
     */
    @ApiModelProperty(value="周期")
    private String period;

    /**
     * 温度
     */
    @ApiModelProperty(value="温度")
    //@Validation(message = "atmosphere_monitor_info表{temp}不能为空")
    private Float temp;

    /**
     * 最低温度
     */
    @ApiModelProperty(value="最低温度")
    private Float lowTemp;

    /**
     * 最高温度
     */
    @ApiModelProperty(value="最高温度")
    private Float highTemp;

    /**
     * 相对湿度
     */
    @ApiModelProperty(value="相对湿度")
    private String sd;

    /**
     * 风速级别
     */
    @ApiModelProperty(value="风速级别")
    private String wse;

    /**
     * 潮流流速
     */
    @ApiModelProperty(value="潮流流速")
    private String flowVelocity;

    /**
     * 潮流流向
     */
    @ApiModelProperty(value="潮流流向")
    private String flowDirection;

    /**
     * 能见度
     */
    @ApiModelProperty(value="能见度")
    private String visibility;

    /**
     * 天气类型
     */
    @ApiModelProperty(value="天气类型")
    private String weatherType;

    /**
     * 极端天气类型
     */
    @ApiModelProperty(value="极端天气类型")
    private String extremeWeatherType;

    /**
     * 监测时间
     */
    @ApiModelProperty(value="监测时间")
    //@Validation(message = "atmosphere_monitor_info表{detection_time}不能为空")
    private LocalDateTime detectionTime;

    @ApiModelProperty(value = "数据收集时间")
    @TableField(fill = FieldFill.INSERT)
    private LocalDateTime createTime;

    @ApiModelProperty(value = "数据更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private LocalDateTime updateTime;

}
