

package com.lutao.entity;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.lutao.annotate.Validation;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.time.LocalDateTime;

/**
 * 安全风险评估信息
 *
 * @author guoyc
 * @date 2023-07-20 10:57:59
 */
@Data
@TableName("security_risk_assessment")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "安全风险评估信息")
public class SecurityRiskAssessment extends Model<SecurityRiskAssessment> {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    @TableId
    @ApiModelProperty(value="主键")
    private Integer id;

    /**
     * 原始id
     */
    @ApiModelProperty(value="原始id")
    @Validation(message = "security_risk_assessment表{id}不能为空")
    private String originId;

    @ApiModelProperty(value = "评估对象类型 1.固定平台，2.钻井平台，3.浮式生产储油装置")
    @Validation(message = "security_risk_assessment表{evaluation_object_type}不能为空")
    private String evaluationObjectType;

    @ApiModelProperty(value = "评估编码")
    @Validation(message = "security_risk_assessment表{evaluation_code}不能为空")
    private String evaluationCode;

    @ApiModelProperty(value = "总扣分数")
    @Validation(message = "security_risk_assessment表{total_deduction}不能为空")
    private Integer totalDeduction;

    @ApiModelProperty(value = "评价类型，1.深评，2.自评")
    @Validation(message = "security_risk_assessment表{evaluation_type}不能为空")
    private String evaluationType;
    /**
     * 得分
     */
    @ApiModelProperty(value="得分")
    @Validation(message = "security_risk_assessment表{score}不能为空")
    private Integer score;

    @ApiModelProperty(value = "风险等级")
    @Validation(message = "security_risk_assessment表{risk_rank}不能为空")
    private String riskRank;

    @ApiModelProperty(value = "整治要求：001高风险平台-全面停产整改\\r\\n002较高风险平台-局部停产整改\\r\\n003中风险平台-限期整改\\r\\n004低风险平台-自行制定计划，落实整改")
    @Validation(message = "security_risk_assessment表{remediation_requirements}不能为空")
    private String remediationRequirements;

    /**
     * 操作人员
     */
    @ApiModelProperty(value="操作人员")
    @Validation(message = "security_risk_assessment表{operator}不能为空")
    private String operator;

    /**
     * 系统平台id
     */
    @ApiModelProperty(value="系统平台id")
    private String pfId;

    /**
     * 应用id
     */
    @ApiModelProperty(value="应用id")
    private String appId;

    /**
     * 平台编码
     */
    @ApiModelProperty(value="平台编码")
    @Validation(message = "security_risk_assessment表{platform_code}不能为空")
    private String platformCode;

    /**
     * 平台名称
     */
    @ApiModelProperty(value="平台名称")
    private String platformName;

    /**
     * 平台id
     */
    @ApiModelProperty(value="平台id")
    private String platformId;

    /**
     * 创建时间
     */
    @ApiModelProperty(value="创建时间")
    private LocalDateTime createDate;

    /**
     * 创建人姓名
     */
    @ApiModelProperty(value="创建人姓名")
    private String createBy;

    /**
     * 修改时间
     */
    @ApiModelProperty(value="修改时间")
    private LocalDateTime updateDate;

    /**
     * 修改人
     */
    @ApiModelProperty(value="修改人")
    private String updateBy;

    @ApiModelProperty(value = "数据收集时间")
    @TableField(fill = FieldFill.INSERT)
    private LocalDateTime createTime;

    @ApiModelProperty(value = "数据更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private LocalDateTime updateTime;
}
