package com.lutao.cloud.admin.service;

import cn.hutool.core.lang.tree.Tree;
import com.baomidou.mybatisplus.extension.service.IService;
import com.lutao.cloud.admin.api.entity.SysMenu;
import com.lutao.cloud.common.core.util.MessageInfo;

import java.util.List;
import java.util.Set;

/**
 * <p>
 * 菜单权限表 服务类
 * </p>
 *

 * @since 2017-10-29
 */
public interface SysMenuService extends IService<SysMenu>{

	/**
	 * 通过角色编号查询URL 权限
	 * @param roleId 角色ID
	 * @return 菜单列表
	 */
	List<SysMenu> findMenuByRoleId(Integer roleId);

	/**
	 * 级联删除菜单
	 * @param id 菜单ID
	 * @return 成功、失败
	 */
	MessageInfo removeMenuById(Integer id);

	/**
	 * 更新菜单信息
	 * @param sysMenu 菜单信息
	 * @return 成功、失败
	 */
	Boolean updateMenuById(SysMenu sysMenu);

	/**
	 * 构建树
	 * @param lazy 是否是懒加载
	 * @param parentId 父节点ID
	 * @return
	 */
	List<Tree<Integer>> treeMenu(boolean lazy, Integer parentId);

	/**
	 * 查询菜单
	 * @param voSet
	 * @param parentId
	 * @return
	 */
	List<Tree<Integer>> filterMenu(Set<SysMenu> voSet, String type, Integer parentId);

	/**
	 * 保存时同步更新父级菜单
	 * @param sysMenu
	 * @return
	 */
	Boolean saveWithParent(SysMenu sysMenu);

	/**
	 * 更新时同步更新父级菜单
	 * @param sysMenu
	 * @return
	 */
	Boolean updateWithParent(SysMenu sysMenu);

	/**
	 * 递归查找父级菜单
	 * @param sysMenu
	 * @param parentSysMenuList
	 * @return
	 */
	List<SysMenu> getParentSysMenu(SysMenu sysMenu, List<SysMenu> parentSysMenuList);

}
